@@ -39,38 +39,36 @@
 
 #![no_std]
 #![doc(
-    html_logo_url = "https://raw.githubusercontent.com/RustCrypto/meta/master/logo.svg",
-    html_favicon_url = "https://raw.githubusercontent.com/RustCrypto/meta/master/logo.svg"
+    html_logo_url = "https://raw.githubusercontent.com/RustCrypto/media/6ee8e381/logo.svg",
+    html_favicon_url = "https://raw.githubusercontent.com/RustCrypto/media/6ee8e381/logo.svg",
+    html_root_url = "https://docs.rs/fsb/0.1.0"
 )]
-#![deny(unsafe_code)]
 #![warn(missing_docs, rust_2018_idioms)]
 #![allow(non_snake_case)]
 
-extern crate alloc;
-
-#[cfg(feature = "std")]
-extern crate std;
-
-use alloc::vec;
-
 #[macro_use]
 mod macros;
 
+use core::fmt;
 pub use digest::{self, Digest};
-use whirlpool::Whirlpool;
-
-use core::convert::TryInto;
 
 // Double check this contains all values in the reference implementation
 static PI: &[u8; 272384] = include_bytes!("pi.bin");
 
-use block_buffer::BlockBuffer;
-use digest::generic_array::GenericArray;
-use digest::{BlockInput, FixedOutputDirty, Reset, Update};
+use digest::{
+    block_buffer::Eager,
+    core_api::{
+        AlgorithmName, Block, BlockSizeUser, Buffer, BufferKindUser, CoreWrapper, FixedOutputCore,
+        OutputSizeUser, Reset, UpdateCore,
+    },
+    generic_array::{typenum::Unsigned, GenericArray},
+    HashMarker, Output,
+};
 
 // FSB-160
 fsb_impl!(
     Fsb160,
+    Fsb160Core,
     160,
     U60,
     U20,
@@ -79,12 +77,14 @@ fsb_impl!(
     640,
     653,
     1120,
-    "FSB-160 hash function."
+    "FSB-160 hasher state",
+    "Core FSB-160 hasher state",
 );
 
 // FSB-224
 fsb_impl!(
     Fsb224,
+    Fsb224Core,
     224,
     U84,
     U28,
@@ -93,12 +93,14 @@ fsb_impl!(
     896,
     907,
     1568,
-    "FSB-224 hash function."
+    "FSB-224 hasher state",
+    "Core FSB-224 hasher state",
 );
 
 // FSB-256
 fsb_impl!(
     Fsb256,
+    Fsb256Core,
     256,
     U96,
     U32,
@@ -107,12 +109,14 @@ fsb_impl!(
     1024,
     1061,
     1792,
-    "FSB-256 hash function."
+    "FSB-256 hasher state",
+    "Core FSB-256 hasher state",
 );
 
 // FSB-384
 fsb_impl!(
     Fsb384,
+    Fsb384Core,
     384,
     U115,
     U48,
@@ -121,12 +125,14 @@ fsb_impl!(
     1472,
     1483,
     2392,
-    "FSB-384 hash function."
+    "FSB-384 hasher state",
+    "Core FSB-384 hasher state",
 );
 
 // FSB-512
 fsb_impl!(
     Fsb512,
+    Fsb512Core,
     512,
     U155,
     U64,
@@ -135,5 +141,6 @@ fsb_impl!(
     1984,
     1987,
     3224,
-    "FSB-512 hash function."
+    "FSB-512 hasher state",
+    "Core FSB-512 hasher state",
 );
